/**********************************************************************
*
*    Author:		shondll (shondll_yahoo.com)
*    Company:		Olimex Ltd.
*    Date:			28/10/2010
*    File Version:	1.00
*    Description:	Initialization routines
*
**********************************************************************/

#include "SystemProfile.h"
#include "Compiler.h"
#include "Console.h"
#include "WirelessProtocols\LCDBlocking.h"
#include "TimeDelay.h"
#include "HardwareProfile.h"
#include "TCN75A.h"

#pragma config FOSC = INTIO67 // Internal oscillator block, port function on RA6 and RA7 
#pragma config FCMEN = OFF // Fail-Safe Clock Monitor disabled  
#pragma config IESO = OFF // Oscillator Switchover mode disabled  
#pragma config PWRT = OFF // PWRT disabled  
#pragma config BOREN = OFF // Brown-out Reset disabled in hardware and software  
#pragma config WDTEN = OFF // WDT is controlled by SWDTEN bit of the WDTCON register  
#pragma config MCLRE = ON // RE3 input pin enabled; MCLR disabled  
#pragma config PBADEN = OFF // PORTB<4:0> pins are configured as digital I/O on Reset  
#pragma config CCP2MX = PORTBE // CCP2 input/output is multiplexed with RB3  
#pragma config LVP = OFF // Single-Supply ICSP disabled  
#pragma config XINST = ON // Instruction set extension and Indexed Addressing mode enabled  
#pragma config DEBUG = ON // Background debugger enabled, RB6 and RB7 are dedicated to In-Circuit Debug
#pragma config WDTPS = 256 // 1:256

#define DEBOUNCE_TIME 0x00003FFF

BOOL PUSH_BUTTON_pressed;
TICK PUSH_BUTTON_press_time;

/*********************************************************************
 * Function:        void BoardInit( void )
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    Board is initialized for P2P usage
 *
 * Overview:        This function configures the board 
 *
 * Note:            This routine needs to be called before the function 
 *                  to initialize P2P stack or any other function that
 *                  operates on the stack
 ********************************************************************/

void BoardInit(void)
{
	OSCCON = 0b01110010; // configure INTERNAL oscillator, 16MHz
	
    WDTCONbits.SWDTEN = 0; //disable WDT
    
    // disable ADC
    ANSEL = 0x0;
    ANSELH = 0x0;
    
    // peripherals
	BUTTON_1_TRIS = 1;
	BUTTON_2_TRIS = 1;
	LED_1_TRIS = 0;
	LED_2_TRIS = 0;
	LED_1 = 0;
	LED_2 = 0;
	
	// setup interrupt on change enable bits
	IOCB = 0b00110000;
	
	// pins of the RF chip
	PHY_CS_TRIS = 0;
	PHY_CS = 1;
	PHY_RESETn_TRIS = 0;
	PHY_RESETn = 1;
	RF_INT_TRIS = 1;
    
    // configure SPI module
	SDI_TRIS = 1;
	SDO_TRIS = 0;
	SCK_TRIS = 0;
	SSPSTAT = 0xC0;
	SSPCON1 = 0x21;
	PHY_WAKE_TRIS = 0;
	PHY_WAKE = 1;
	INTCON2bits.INTEDG0 = 0;
	
	// configure HLVD module (does not enable it!)
	HLVDCON = 0b00001001; // trip at 2.8V
	
	
	// enable interrupts
	INTCONbits.GIEH = 1;
	
	// enable interrupts by the rf chip
	RFIF = 0;
	RFIE = 1;
	
	TRISA &= ~0b11110000;
	TRISB &= ~0b11001100;
	TRISC &= ~0b11000000;
}

/*********************************************************************
 * Function:        BYTE ButtonPressed(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          Byte to indicate which button has been pressed. 
 *                  Return 0 if no button pressed.
 *
 * Side Effects:    
 *
 * Overview:        This function check if any button has been pressed
 *
 * Note:            
 ********************************************************************/
BYTE ButtonPressed(void)
{
    TICK tickDifference;
        
    if(PUSH_BUTTON_1 == 0)
    {
        //if the button was previously not pressed
        if(PUSH_BUTTON_pressed == FALSE)
        {
            PUSH_BUTTON_pressed = TRUE;
            PUSH_BUTTON_press_time = TickGet();
            return 1;
        }
    }
    else if(PUSH_BUTTON_2 == 0)
    {
        //if the button was previously not pressed
        if(PUSH_BUTTON_pressed == FALSE)
        {
            PUSH_BUTTON_pressed = TRUE;
            PUSH_BUTTON_press_time = TickGet();
            return 2;
        }
    } 
    else
    {
        //get the current time
        TICK t = TickGet();
        
        //if the button has been released long enough
        tickDifference.Val = TickGetDiff(t,PUSH_BUTTON_press_time);
        
        //then we can mark it as not pressed
        if(tickDifference.Val > DEBOUNCE_TIME)
        {
            PUSH_BUTTON_pressed = FALSE;
        }
    }
    
    return 0;
}

//void I2CTask(void);

void UserInterruptHandler(void)
{
	static BYTE dummy;
	
//	// timer1 interrupt
//	if( PIR1bits.TMR1IF ) {
//		PIR1bits.TMR1IF = 0;
//		
//		// juct leave, all we wanted was to wake up the device
//		
//		LED_2 ^= 1; 
//	}

	// pin state change has to only wake up the device, nothimg more to do
	if(INTCONbits.RBIF) {
		INTCONbits.RBIE = 0;        // disable pin change notification interrupt
		
		// flag bit is cleared after being ckecked
	}
}              
